"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.GitHub = void 0;
const Logger_1 = require("../../Logger");
const Parser_1 = require("../../Parser");
const filter_util_1 = require("../utils/filter.util");
const message_util_1 = require("../utils/message.util");
const CommitStatus_1 = require("./CommitStatus");
class GitHub {
    constructor(prService, removeOldComment = false) {
        this.prService = prService;
        this.removeOldComment = removeOldComment;
    }
    report(logs) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield this.setup(logs);
                if (this.removeOldComment) {
                    yield this.removeExistingComments();
                }
                yield Promise.all(this.comments.map((c) => this.createReviewComment(c)));
                yield this.createSummaryComment();
                yield this.setCommitStatus();
                Logger_1.Log.info('Report commit status completed');
            }
            catch (err) {
                Logger_1.Log.error('GitHub report failed', err);
                throw err;
            }
        });
    }
    createSummaryComment() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.nWarning + this.nError > 0 || this.invalidLogs.length > 0) {
                const overview = message_util_1.MessageUtil.generateOverviewMessage(this.nError, this.nWarning);
                const other = message_util_1.MessageUtil.createOtherIssueReport(this.invalidLogs);
                yield this.prService.createComment(overview + (other ? `\n${other}` : ''));
                Logger_1.Log.info('Create summary comment completed');
            }
            else {
                Logger_1.Log.info('No summary comment needed');
            }
        });
    }
    setCommitStatus() {
        return __awaiter(this, void 0, void 0, function* () {
            const commitStatus = this.nError > 0 ? CommitStatus_1.CommitStatus.failure : CommitStatus_1.CommitStatus.success;
            const description = message_util_1.MessageUtil.generateCommitDescription(this.nError);
            yield this.prService.setCommitStatus(this.commitId, commitStatus, description);
        });
    }
    setup(logs) {
        return __awaiter(this, void 0, void 0, function* () {
            this.commitId = yield this.prService.getLatestCommitSha();
            this.touchedDiff = yield this.prService.diff();
            this.invalidLogs = logs.filter((l) => !l.valid);
            const touchedFileLog = logs
                .filter(filter_util_1.onlySeverity(Parser_1.LogSeverity.error, Parser_1.LogSeverity.warning))
                .filter(filter_util_1.onlyIn(this.touchedDiff));
            this.comments = GitHub.groupComments(touchedFileLog);
            this.nError = this.comments.reduce((sum, comment) => sum + comment.errors, 0);
            this.nWarning = this.comments.reduce((sum, comment) => sum + comment.warnings, 0);
            Logger_1.Log.debug(`VCS Setup`, {
                sha: this.commitId,
                diff: this.touchedDiff,
                comments: this.comments,
                err: this.nError,
                warning: this.nWarning,
            });
        });
    }
    createReviewComment(comment) {
        return __awaiter(this, void 0, void 0, function* () {
            const { text, file, line } = comment;
            yield this.prService.createReviewComment(this.commitId, text, file, line);
            Logger_1.Log.debug('GitHub create review success', { text, file, line });
            return comment;
        });
    }
    removeExistingComments() {
        return __awaiter(this, void 0, void 0, function* () {
            const [userId, comments, reviews] = yield Promise.all([
                this.prService.getCurrentUserId(),
                this.prService.listAllComments(),
                this.prService.listAllReviewComments(),
            ]);
            Logger_1.Log.debug('Get existing CodeCoach comments completed');
            const deleteComments = comments
                .filter((comment) => { var _a; return ((_a = comment.user) === null || _a === void 0 ? void 0 : _a.id) === userId; })
                .map((comment) => this.prService.deleteComment(comment.id));
            const deleteReviews = reviews
                .filter((review) => { var _a; return ((_a = review.user) === null || _a === void 0 ? void 0 : _a.id) === userId; })
                .map((review) => this.prService.deleteReviewComment(review.id));
            yield Promise.all([...deleteComments, ...deleteReviews]);
            Logger_1.Log.debug('Delete CodeCoach comments completed');
        });
    }
    static groupComments(logs) {
        const commentMap = logs.reduce((map, log) => {
            var _a, _b, _c, _d, _e, _f, _g, _h, _j;
            const { source: file, line, severity, msg } = log;
            const text = message_util_1.MessageUtil.createMessageWithEmoji(msg, severity);
            if (!line)
                return map;
            const currentWarnings = (_c = (_b = (_a = map === null || map === void 0 ? void 0 : map[file]) === null || _a === void 0 ? void 0 : _a[line]) === null || _b === void 0 ? void 0 : _b.warnings) !== null && _c !== void 0 ? _c : 0;
            const currentErrors = (_f = (_e = (_d = map === null || map === void 0 ? void 0 : map[file]) === null || _d === void 0 ? void 0 : _d[line]) === null || _e === void 0 ? void 0 : _e.errors) !== null && _f !== void 0 ? _f : 0;
            const currentText = (_j = (_h = (_g = map === null || map === void 0 ? void 0 : map[file]) === null || _g === void 0 ? void 0 : _g[line]) === null || _h === void 0 ? void 0 : _h.text) !== null && _j !== void 0 ? _j : '';
            const nextObject = {
                text: `${currentText}\n${text}`,
                errors: currentErrors + (severity === Parser_1.LogSeverity.error ? 1 : 0),
                warnings: currentWarnings + (severity === Parser_1.LogSeverity.warning ? 1 : 0),
                file,
                line,
            };
            const fileCommentUpdater = { [line]: nextObject };
            const updatedFileComment = Object.assign({}, map === null || map === void 0 ? void 0 : map[file], fileCommentUpdater);
            const mapUpdater = { [file]: updatedFileComment };
            return Object.assign({}, map, mapUpdater);
        }, {});
        return Object.values(commentMap).flatMap((file) => Object.values(file));
    }
}
exports.GitHub = GitHub;
