"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.GitHubPRService = void 0;
const rest_1 = require("@octokit/rest");
const url_1 = require("url");
const app_constants_1 = require("../../app.constants");
const defaults_1 = require("../../Config/constants/defaults");
const Logger_1 = require("../../Logger");
const github_provider_constant_1 = require("../constants/github.provider.constant");
const patchProcessor_1 = require("../utils/patchProcessor");
class GitHubPRService {
    constructor(token, repoUrl, pr) {
        this.pr = pr;
        const repoUrlObj = new url_1.URL(repoUrl);
        this.adapter = new rest_1.Octokit({
            auth: token,
            userAgent: defaults_1.USER_AGENT,
            timeZone: defaults_1.TIME_ZONE,
            baseUrl: GitHubPRService.getApiBase(repoUrlObj),
        });
        const [, owner, repo] = repoUrlObj.pathname.replace(/\.git$/gi, '').split('/');
        this.requestBase = { owner, repo };
    }
    listAllReviewComments() {
        return __awaiter(this, void 0, void 0, function* () {
            const reviews = [];
            let page = 0;
            let hasNext = true;
            while (hasNext) {
                const { headers, data } = yield this.adapter.pulls.listReviewComments(Object.assign(Object.assign({}, this.requestBase), { pull_number: this.pr, per_page: github_provider_constant_1.API_PAGE_SIZE_LIMIT, page: ++page }));
                reviews.push(...data);
                hasNext = GitHubPRService.hasNext(headers);
            }
            Logger_1.Log.debug(`Loaded ${page} pages of reviews`);
            return reviews;
        });
    }
    listAllComments() {
        return __awaiter(this, void 0, void 0, function* () {
            const comments = [];
            let page = 0;
            let hasNext = true;
            while (hasNext) {
                const { headers, data } = yield this.adapter.issues.listComments(Object.assign(Object.assign({}, this.requestBase), { issue_number: this.pr, per_page: github_provider_constant_1.API_PAGE_SIZE_LIMIT, page: ++page }));
                comments.push(...data);
                hasNext = GitHubPRService.hasNext(headers);
            }
            Logger_1.Log.debug(`Loaded ${page} pages of comments`);
            return comments;
        });
    }
    deleteComment(comment_id) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.adapter.issues.deleteComment(Object.assign(Object.assign({}, this.requestBase), { comment_id }));
        });
    }
    createComment(body) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.adapter.issues.createComment(Object.assign(Object.assign({}, this.requestBase), { issue_number: this.pr, body }));
        });
    }
    deleteReviewComment(comment_id) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.adapter.pulls.deleteReviewComment(Object.assign(Object.assign({}, this.requestBase), { comment_id }));
        });
    }
    createReviewComment(commit_id, body, path, line) {
        return __awaiter(this, void 0, void 0, function* () {
            const commonParams = Object.assign(Object.assign({ mediaType: {
                    previews: ['comfort-fade'],
                } }, this.requestBase), { pull_number: this.pr, body,
                commit_id,
                path });
            yield this.adapter.pulls.createReviewComment(line ? Object.assign(Object.assign({}, commonParams), { line, side: 'RIGHT' }) : Object.assign(Object.assign({}, commonParams), { position: 1 }));
        });
    }
    getCurrentUserId() {
        return __awaiter(this, void 0, void 0, function* () {
            const response = yield this.adapter.users.getAuthenticated();
            const user = response.data;
            return user.id;
        });
    }
    getLatestCommitSha() {
        return __awaiter(this, void 0, void 0, function* () {
            const { data } = yield this.adapter.pulls.get(Object.assign(Object.assign({}, this.requestBase), { pull_number: this.pr }));
            return data.head.sha;
        });
    }
    setCommitStatus(sha, state, description) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield this.adapter.repos.createCommitStatus(Object.assign(Object.assign({}, this.requestBase), { sha,
                    state,
                    description, context: 'CodeCoach' }));
            }
            catch (err) {
                if (err.name === 'HttpError') {
                    Logger_1.Log.error('NotFound: Could be repository does not exist or token does not have permission to repository');
                }
                throw err;
            }
        });
    }
    diff() {
        return __awaiter(this, void 0, void 0, function* () {
            const files = [];
            let page = 0;
            let hasNext = true;
            while (hasNext) {
                const { headers, data } = yield this.adapter.pulls.listFiles(Object.assign(Object.assign({}, this.requestBase), { pull_number: this.pr, per_page: github_provider_constant_1.API_PAGE_SIZE_LIMIT, page: ++page }));
                files.push(...data.map((d) => ({ file: d.filename, patch: patchProcessor_1.getPatch(d.patch) })));
                hasNext = GitHubPRService.hasNext(headers);
            }
            Logger_1.Log.debug(`Loaded ${page} pages of PR file list`);
            return files;
        });
    }
    static hasNext(headers) {
        var _a;
        const { link } = headers;
        return ((_a = link === null || link === void 0 ? void 0 : link.split(', ').map((l) => { var _a; return (_a = l.match(/rel="(.+)"/)) === null || _a === void 0 ? void 0 : _a[1]; }).some((rel) => rel === 'next')) !== null && _a !== void 0 ? _a : false);
    }
    static getApiBase(repo) {
        return repo.hostname === 'github.com'
            ? app_constants_1.GITHUB_COM_API
            : new url_1.URL('api/v3', repo.origin).toString();
    }
}
exports.GitHubPRService = GitHubPRService;
