#!/usr/bin/env node
"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Config_1 = require("./Config");
const File_1 = require("./File");
const Logger_1 = require("./Logger");
const Parser_1 = require("./Parser");
const Provider_1 = require("./Provider");
const DartLintParser_1 = require("./Parser/DartLintParser");
class App {
    constructor() {
        const githubPRService = new Provider_1.GitHubPRService(Config_1.Config.provider.token, Config_1.Config.provider.repoUrl, Config_1.Config.provider.prId);
        this.vcs = new Provider_1.GitHub(githubPRService, Config_1.Config.provider.removeOldComment);
    }
    start() {
        return __awaiter(this, void 0, void 0, function* () {
            const logs = yield this.parseBuildData(Config_1.Config.app.buildLogFiles);
            Logger_1.Log.info('Build data parsing completed');
            // Fire and forget, no need to await
            App.writeLogToFile(logs)
                .then(() => Logger_1.Log.info('Write output completed'))
                .catch((error) => Logger_1.Log.error('Write output failed', { error }));
            yield this.vcs.report(logs);
            Logger_1.Log.info('Report to VCS completed');
        });
    }
    static getParser(type, cwd) {
        switch (type) {
            case Config_1.ProjectType.dotnetbuild:
                return new Parser_1.DotnetBuildParser(cwd);
            case Config_1.ProjectType.msbuild:
                return new Parser_1.MSBuildParser(cwd);
            case Config_1.ProjectType.tslint:
                return new Parser_1.TSLintParser(cwd);
            case Config_1.ProjectType.eslint:
                return new Parser_1.ESLintParser(cwd);
            case Config_1.ProjectType.scalastyle:
                return new Parser_1.ScalaStyleParser(cwd);
            case Config_1.ProjectType.androidlint:
                return new Parser_1.AndroidLintStyleParser(cwd);
            case Config_1.ProjectType.dartlint:
                return new DartLintParser_1.DartLintParser(cwd);
        }
    }
    parseBuildData(files) {
        return __awaiter(this, void 0, void 0, function* () {
            const logsTasks = files.map(({ type, path, cwd }) => __awaiter(this, void 0, void 0, function* () {
                Logger_1.Log.debug('Parsing', { type, path, cwd });
                const content = yield File_1.File.readFileHelper(path);
                const parser = App.getParser(type, cwd);
                return parser.parse(content);
            }));
            return (yield Promise.all(logsTasks)).flatMap((x) => x);
        });
    }
    static writeLogToFile(logs) {
        return __awaiter(this, void 0, void 0, function* () {
            yield File_1.File.writeFileHelper(Config_1.Config.app.logFilePath, JSON.stringify(logs, null, 2));
        });
    }
}
new App().start().catch((error) => {
    if (error instanceof Error) {
        const { stack, message } = error;
        Logger_1.Log.error('Unexpected error', { stack, message });
    }
    Logger_1.Log.error('Unexpected error', { error });
    process.exit(1);
});
